﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Linq;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Net.NetworkInformation;
using SciLors_DHCP_Reset_Config;
using System.Xml;
using System.Xml.Serialization;
using SciLors_DHCP_Reset;
using System.Diagnostics;
using IWshRuntimeLibrary;

namespace SciLor_s_DHCP_Reset_Config {
    public partial class frmMain : Form {
        private static NetworkInterfaceType[] nit_array = { 
            NetworkInterfaceType.Ethernet,
            NetworkInterfaceType.Ethernet3Megabit,
            NetworkInterfaceType.FastEthernetFx,
            NetworkInterfaceType.FastEthernetT,
            NetworkInterfaceType.GigabitEthernet,
            NetworkInterfaceType.Wireless80211
                                                          };

        private List<NetworkInterfaceType> nit = nit_array.ToList();

        public frmMain() {
            InitializeComponent();
        }

        private void ConfigTask(String setting) {
            Process p = new Process();
            p.StartInfo.FileName = "SciLors DHCP Reset.exe";
            p.StartInfo.Arguments = setting;
            p.StartInfo.Verb = "runas";
            p.StartInfo.WindowStyle = ProcessWindowStyle.Normal;
            p.Start();
            p.WaitForExit();
        }

        private void frmMain_Load(object sender, EventArgs e) {
            clbNetworkCards.Items.Clear();
            foreach (NetworkInterface n in NetworkInterface.GetAllNetworkInterfaces()) {
                if (nit.Contains(n.NetworkInterfaceType)) 
                    clbNetworkCards.Items.Add(new NetworkCard(n));
            }
            LoadCfg();
            btnTask_SetText();
        }

        private void Save() {
            List<String> nl = new List<String>();
            for (int i = 0; i < clbNetworkCards.Items.Count; i++) {
                NetworkCard nc = (NetworkCard)clbNetworkCards.Items[i];
                if (nc.reset) nl.Add(nc.name);
            }
            Config.Save(nl);
        }
        private void LoadCfg() {
            List<String> nl = new List<String>();
            nl = Config.Load();

            for (int i = 0; i < clbNetworkCards.Items.Count; i++) {
                NetworkCard nc = (NetworkCard)clbNetworkCards.Items[i];
                if (nl.Contains(nc.name)) {
                    clbNetworkCards.SetItemChecked(i, true);
                }
            }
        }

        private void clbNetworkCards_ItemCheck(object sender, ItemCheckEventArgs e) {
            if (e.Index == -1) return;
            if (e.NewValue == CheckState.Checked) {
                (clbNetworkCards.Items[e.Index] as NetworkCard).reset = true;
            } else {
                (clbNetworkCards.Items[e.Index] as NetworkCard).reset = false;
            }
        }

        private void frmMain_FormClosing(object sender, FormClosingEventArgs e) {
            Save();
        }

        private void btnTask_SetText() {
            if (SciLor_s_DHCP_Reset.Program.taskExists()) {
                btnTask.Text = "Remove Task";
                grpTask.Enabled = false;
                btnTaskLink.Enabled = true;
            } else {
                btnTask.Text = "Install as Task";
                grpTask.Enabled = true;
                btnTaskLink.Enabled = false;
            }
        }

        private String chkToString(CheckBox chk) {
            if (chk.Checked) return SciLor_s_DHCP_Reset.Program.ENABLED;
            return SciLor_s_DHCP_Reset.Program.DISABLED;
        }

        private void btnTask_Click(object sender, EventArgs e) {
            if (SciLor_s_DHCP_Reset.Program.taskExists()) {
                ConfigTask(SciLor_s_DHCP_Reset.Program.REMOVE_TASK);
            } else {
                String config = SciLor_s_DHCP_Reset.Program.INSTALL_AS_TASK + " " +
                    chkToString(chkTaskLogon) + " " +
                    chkToString(chkTaskWakeUp);
                ConfigTask(config);
            }
            btnTask_SetText();
        }

        private void btnTaskLink_Click(object sender, EventArgs e) {
            SaveFileDialog sfd = new SaveFileDialog();
            sfd.DefaultExt = ".lnk";
            sfd.FileName = "SciLors DHCP Reset (No UAC).lnk";
            sfd.Filter = "Shortcut (*.lnk)|*.lnk";
            sfd.ValidateNames = true;
            if (sfd.ShowDialog() == System.Windows.Forms.DialogResult.OK && sfd.FileName != null) {
                WshShellClass wshShell = new WshShellClass();
                IWshShortcut link = (IWshShortcut)(wshShell.CreateShortcut(sfd.FileName));
                link.TargetPath = "schtasks.exe";
                link.Arguments = "/run /tn \"" + SciLor_s_DHCP_Reset.Program.TASK_NAME + "\"";
                link.Description = "Starts SciLor's DHCP Reset without an UAC Prompt";
                link.WindowStyle = 7;
                link.Save();
            }
        }

    }
}
