﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Diagnostics;
using SciLors_DHCP_Reset;
using Microsoft.Win32.TaskScheduler;

namespace SciLor_s_DHCP_Reset {
    public class Program {
        public const String ENABLED = "enabled";
        public const String DISABLED = "disabled";
        public const String INSTALL_AS_TASK = "-installastask";
        public const String REMOVE_TASK = "-removetask";
        public const String TASK_NAME = "SciLor's DHCP Reset";

        static void Main(string[] args) {
            if (args.Length > 0) {
                if (args.Length == 3 && args[0] == INSTALL_AS_TASK) {
                    asTask(args[1] == ENABLED, args[2] == ENABLED);
                    return;
                } else if (args[0] == REMOVE_TASK) {
                    removeTask();
                    return;
                } 
            }

            List<String> adapter = Config.Load();
            foreach (String adpt in adapter) {
                resetIp(adpt);
                resetDns(adpt);
            }
        }


        private static void asTask(bool logon, bool wakeup) {
            TaskService ts = new TaskService();

            TaskDefinition td = ts.NewTask();

            //td.Triggers.Add(new DailyTrigger { DaysInterval = 2 });
            //td.Triggers.Add(new EventTrigger("System", "Kernel-Power", 42); //Sleep
            if (wakeup) td.Triggers.Add(new EventTrigger("System", "Microsoft-Windows-Power-Troubleshooter", 1)); //Wakeup
            if (logon) td.Triggers.Add(new LogonTrigger());

            td.Actions.Add(new ExecAction("\"" + GetExePath() + "\"", null, System.IO.Path.GetDirectoryName(GetExePath())));

            td.Settings.DisallowStartIfOnBatteries = false;
            td.Principal.RunLevel = TaskRunLevel.Highest;
            td.Settings.Hidden = true;
            ts.RootFolder.RegisterTaskDefinition(TASK_NAME, td);
        }
        public static Boolean taskExists() {
            TaskService ts = new TaskService();
            foreach (Task task in ts.RootFolder.GetTasks()) {
                if (task.Name == TASK_NAME) {
                    return true;
                }
            }
            return false;
        }
        private static void removeTask() {
            TaskService ts = new TaskService();
            try {
                ts.RootFolder.DeleteTask(TASK_NAME);
            } catch (Exception) { }
        }

        private static String GetExePath() {
            String path = System.Reflection.Assembly.GetExecutingAssembly().CodeBase;
            if (path.StartsWith("file:///")) {
                path = path.Substring("file:///".Length);
            }
            path = path.Replace('/', '\\');
            return path;
        }

        private static void createNetShAdminProcess(String args) {
            Process p = new Process();
            p.StartInfo.FileName = "netsh";
            p.StartInfo.Arguments = args;
            p.StartInfo.Verb = "runas";
            p.StartInfo.WindowStyle = ProcessWindowStyle.Hidden;
            p.Start();
        }

        private static void resetIp(String name) {
            createNetShAdminProcess("interface ip set address \"" + name + "\" dhcp");
        }
        private static void resetDns(String name) {
            createNetShAdminProcess("interface ip set dns \"" + name + "\" dhcp");
        }
    }
}
